@@ -8,21 +8,22 @@ use AnyEvent::Socket;
 use AnyEvent::TLS;
 
 require bytes;
+use Carp qw(croak);
 use Encode;
 use Scalar::Util 'looks_like_number';
 use JSON::Any;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 has certificate => (
     is       => 'rw',
-    isa      => 'Str',
+    isa      => 'Str | ScalarRef',
     required => 1,
 );
 
 has private_key => (
     is       => 'rw',
-    isa      => 'Str',
+    isa      => 'Str | ScalarRef',
     required => 1,
 );
 
@@ -33,8 +34,8 @@ has sandbox => (
 );
 
 has handler => (
-    is      => 'rw',
-    isa     => 'AnyEvent::Handle',
+    is        => 'rw',
+    isa       => 'AnyEvent::Handle',
     predicate => 'connected',
     clearer   => 'clear_handler',
 );
@@ -145,11 +146,25 @@ sub connect {
         $host = '127.0.0.1';
         $port = $self->debug_port;
     }
-
     my $g = tcp_connect $host, $port, sub {
         my ($fh) = @_
             or return $self->on_error->(undef, 1, $!);
 
+        my $tls_setting = {};
+        if (ref $self->certificate) {
+            $tls_setting->{cert}      = ${ $self->certificate };
+        }
+        else {
+            $tls_setting->{cert_file} = $self->certificate;
+        }
+
+        if (ref $self->private_key) {
+            $tls_setting->{key}       = ${ $self->private_key };
+        }
+        else {
+            $tls_setting->{key_file}  = $self->private_key;
+        }
+
         my $handle = AnyEvent::Handle->new(
             fh       => $fh,
             on_error => sub {
@@ -159,10 +174,7 @@ sub connect {
             },
             !$self->is_debug ? (
                 tls      => 'connect',
-                tls_ctx  => {
-                    cert_file => $self->certificate,
-                    key_file  => $self->private_key,
-                },
+                tls_ctx  => $tls_setting,
             ) : (),
         );
         $self->handler( $handle );
@@ -243,13 +255,21 @@ Supported arguments are:
 
 =over 4
 
-=item certificate => 'your apns certificate file'
+=item certificate => 'Str | ScalarRef'
 
-Required
+    certificate => '/path/to/certificate_file',
+    # or
+    certificate => \$certificate,
 
-=item private_key => 'your apns private key file',
+Required. Either file path for certificate or scalar-ref of certificate data.
 
-Required
+=item private_key => 'Str | ScalarRef'
+
+    private_key => '/path/to/private_key',
+    # or
+    private_key => \$private_key,
+
+Required. Either file path for private_key or scalar-ref of private-key data.
 
 =item sandbox => 0|1
 
